
use crate::backend_piston::SchemaPiston;
use crate::{EventType, ToEventType};
use crate::types::{InputTypeFlags, InputCause, ButtonState};
use piston::input::{Event, Input, Button, ButtonArgs, Key, ButtonState as PButtonState, ControllerButton, Motion};

fn new_event_button(button: Button, state: PButtonState) -> Event {
    Event::Input(Input::Button(ButtonArgs {
        button: button,
        scancode: Some(0x20),
        state: state,
    }))
}

#[test]
fn event_to_raw() {
    let event = new_event_button(Button::Keyboard(Key::Space), PButtonState::Press);
    let event_ignorable = Event::Input(Input::Cursor(false));

    assert_eq!(event.to_raw(), Some(EventType((1 as u128).rotate_left(64) + (Key::Space as u128))));
    assert_eq!(event_ignorable.to_raw(), None);   
}

#[test]
fn event_controller_id() {
    let event = new_event_button(Button::Controller(ControllerButton::new(5, 1)), PButtonState::Press);
    let event_ignorable = Event::Input(Input::Cursor(false));
    assert_eq!(event.controller_id(), Some(5));
    assert_eq!(event_ignorable.controller_id(), None);
}

#[test]
fn event_filter_for_assignment() {
    let event = new_event_button(Button::Controller(ControllerButton::new(5, 1)), PButtonState::Press);
    let event_other = Event::Input(Input::Cursor(false));

    assert_eq!(event.filter_for_assignment(InputTypeFlags::Axis), false);
    assert_eq!(event.filter_for_assignment(InputTypeFlags::Key), false);
    assert_eq!(event.filter_for_assignment(InputTypeFlags::Button), true);
    assert_eq!(event_other.filter_for_assignment(InputTypeFlags::Other), true);
}

#[test]
fn event_get_cause() {
    let event = new_event_button(Button::Controller(ControllerButton::new(5, 1)), PButtonState::Press);
    let event_other = Event::Input(Input::Cursor(false));
    let event_mouse = Event::Input(Input::Move(Motion::MouseCursor(0.4, 0.5)));

    assert_eq!(event.get_cause(), Some(InputCause::Button(ButtonState::Pressed)));
    assert_eq!(event_other.get_cause(), None);
    assert_eq!(event_mouse.get_cause(), Some(InputCause::Motion(0.4, 0.5)));
}

#[test]
fn schema_set_player_id() {
    let mut schema = SchemaPiston::<u64>::new("Testing");

    assert_eq!(schema.player_id(), None);
    schema.set_player_id(Some(5));
    assert_eq!(schema.player_id(), Some(5));
    schema.set_player_id(None);
    assert_eq!(schema.player_id(), None);
}

#[test]
fn schema_clear_assignments() {
    let mut schema = SchemaPiston::<u64>::new("Testing");

    assert_eq!(schema.player_id(), None);
    schema.clear_assignments();
    assert_eq!(schema.player_id(), None);

    schema.set_player_id(Some(5));
    schema.clear_assignments();
    assert_eq!(schema.player_id(), Some(5));
}
