use crate::{EventType, ToEventType, Schema};

// required for schema instantiations
impl ToEventType<usize> for u64
{
    fn to_raw(&self) -> Option<EventType> {
        Some(EventType::new(*self, *self))
    }

    fn controller_id(&self) -> Option<usize> {
        if *self > 0 { // we can test both variants in schema tests this way
            return Some(*self as usize)
        }

        None
    }

    fn filter_for_assignment(&self, iaf: crate::types::InputTypeFlags) -> bool {
        iaf == crate::types::InputTypeFlags::Other
    }

    fn get_cause(&self) -> Option<crate::types::InputCause> {
        match *self {
            1 => Some(crate::types::InputCause::Button(crate::types::ButtonState::Pressed)),
            2 => Some(crate::types::InputCause::Button(crate::types::ButtonState::Released)),
            3 => Some(crate::types::InputCause::Axis(0, 0.0)),
            4 => Some(crate::types::InputCause::Axis(0, 1.0)),
            5 => Some(crate::types::InputCause::Motion(0.0, 0.0)),
            6 => Some(crate::types::InputCause::Motion(1.0, 1.0)),
            103 => Some(crate::types::InputCause::Motion(0.2, 0.5)),
            _ => None,
        }
    }
}

#[test]
fn event_type_new() {
    let rt: u64 = 348;
    let rv: u64 = 982374;
    assert!(EventType::new(rt, rv).0 == (rt as u128).rotate_left(64) + u128::from(rv));
}

#[test]
fn schema_new() {
    assert!(Schema::<u64, usize, usize>::new("somename").name == "somename");
}

#[test]
fn schema_assign_controller() {
    let mut schema = Schema::<u64, usize, usize>::new("somename");

    assert!(schema.assign_controller(&0, crate::types::InputTypeFlags::Any) == false);
    assert!(schema.assign_controller(&1, crate::types::InputTypeFlags::Other) == true);
    assert!(schema.controller_id == Some(1));
    assert!(schema.assign_controller(&2, crate::types::InputTypeFlags::Other) == true);
    assert!(schema.controller_id == Some(2));
    assert!(schema.assign_controller(&2, crate::types::InputTypeFlags::Other) == false);
    assert!(schema.controller_id == Some(2));
}

fn schema_assign_input_internal(schema: &mut Schema<u64, usize, usize>) {
    assert_eq!(schema.assign_input(&100, 3, crate::types::InputTypeFlags::Any), false); // filtered out (InputTypeFlags::Any -> ignore)
    assert_eq!(schema.assign_input(&110, 0, crate::types::InputTypeFlags::Other), true); // empty cause
    assert_eq!(schema.assign_input(&101, 4, crate::types::InputTypeFlags::Other), true); // 0/1.0 axis
    assert_eq!(schema.assign_input(&102, 1, crate::types::InputTypeFlags::Other), true); // release button 
    assert_eq!(schema.assign_input(&103, 5, crate::types::InputTypeFlags::Other), true); // 0.0/0.0 motion
    assert_eq!(schema.keymap.len(), 4);
    assert_eq!(schema.keymap.get(&EventType::new(110, 110)), Some(&0));
    assert_eq!(schema.keymap.get(&EventType::new(101, 101)), Some(&4));
    assert_eq!(schema.keymap.get(&EventType::new(102, 102)), Some(&1));
    assert_eq!(schema.keymap.get(&EventType::new(103, 103)), Some(&5));

    assert_eq!(schema.assign_input(&103, 6, crate::types::InputTypeFlags::Other), true); // override with motion 1.0/1.0
    assert_eq!(schema.keymap.get(&EventType::new(103, 103)), Some(&6));
}

#[test]
fn schema_assign_input() {
    let mut schema = Schema::<u64, usize, usize>::new("somename");
    schema_assign_input_internal(&mut schema);
}

#[test]
fn schema_process_event() {
    let mut schema = Schema::<u64, usize, usize>::new("somename");
    schema_assign_input_internal(&mut schema);
    
    // NOTE: last assign set the controller_id to "103" so this is the only one that passes here
    assert_eq!(schema.process_event(&103), Some(crate::types::ProcessingResult::<usize> {
        action: 6,
        player_id: None,
        cause: Some(crate::types::InputCause::Motion(0.2, 0.5)),
    }));

    // doesn't pass since controller_id for event != controller_id set for schema
    assert_eq!(schema.process_event(&102), None);
}

#[test]
fn schema_set_player_id() {
    let mut schema = Schema::<u64, usize, usize>::new("somename");

    schema.set_player_id(Some(5));
    assert!(schema.player_id == Some(5));
    schema.set_player_id(Some(34258));
    assert!(schema.player_id == Some(34258));
}